#   EdgeLengths.py
#       created on: 24 January 2017
#
#           Author: Andreas Gadin

import numpy
import sys
import os

# This script uses two files as input, the a trajectory-file .conf and a file consisting of the indices for particles,
# two per row, that are used to retrieve the positions corresponding to those indices from the trajectory-file.
# The distances are the calculated by extracting two positions (corresponding to the two indices from every row in the
#  second file) then the norm is applied to the difference of them for every frame in the trajectoryfile.

class EdgeLength:

    # Read a file into memory and returns is as a list of lines.
    @staticmethod
    def read(filename):
        with open(filename, 'r') as f:
            lines = f.readlines()
        return lines

    # returns an mx2 list of floats where m is the number of edges
    @staticmethod
    def splitParticles(particle_ids):
        particle_list = []
        for line in particle_ids:
            line = line.split(' ')
            particle_list.append([int(line[2]), int(line[3])])
        return particle_list

    # Returns the total number of frames and the total number of particles in the trajectoryfile.
    @staticmethod
    def frames(lines):
        counter = 0
        for line in lines:
            line.split(' ')
            if line[0] == 't':
                counter += 1
        np = int((len(lines) - 3 * counter) / counter)
        print(np)
        return np, counter

    # Uses the list of particles returned from splitParticles() and uses that to get the positions from the trajectory.
    # Then the distances are calculated and returned as a list.
    @staticmethod
    def getEdgeDistance(particle_list, trajectory):

        np, frames_nr = EdgeLength.frames(trajectory)
        counter = 3
        distances = []
        for j in enumerate(particle_list):
            distances.append([])
            for i in range(0, frames_nr):
                distances[-1].append([])

        frame = 0
        for i in range(0, frames_nr):
            for j, particles in enumerate(particle_list):
                p1_id = particles[0]
                p2_id = particles[1]
                line = trajectory[i * np + p1_id + counter]
                line = line.split(" ")
                p1 = numpy.array([float(line[0]), float(line[1]), float(line[2])])
                line = trajectory[i * np + p2_id + counter]
                line = line.split(" ")
                p2 = numpy.array([float(line[0]), float(line[1]), float(line[2])])
                d = numpy.linalg.norm(p1 - p2)
                distances[j][frame] = d
            frame += 1
            counter += 3
        return distances

    @staticmethod
    def getEndCoordinates(particle_list, trajectory):# Based on getEdgeDistance, this function extracts the end base coordinates in all edges in all frames

        np, frames_nr = EdgeLength.frames(trajectory)
        counter = 3
        distances = []
        for j in enumerate(particle_list):#for every edge i need 6 list: x,y,z for the end coordinates
            for _ in range(6):
                distances.append([])
                for i in range(0, frames_nr):
                    distances[-1].append([])

        frame = 0
        for i in range(0, frames_nr):
            for j, particles in enumerate(particle_list):
                p1_id = particles[0]
                p2_id = particles[1]
                line = trajectory[i * np + p1_id + counter]
                line = line.split(" ")
                p1 = numpy.array([float(line[0]), float(line[1]), float(line[2])])#extract the coordinates of the end particles
                line = trajectory[i * np + p2_id + counter]
                line = line.split(" ")
                p2 = numpy.array([float(line[0]), float(line[1]), float(line[2])])

                distances[j*6][frame] = p1[0]#store them to a list
                distances[(j*6)+1][frame] = p1[1]
                distances[(j*6)+2][frame] = p1[2]
                distances[(j*6)+3][frame] = p2[0]
                distances[(j*6)+4][frame] = p2[1]
                distances[(j*6)+5][frame] = p2[2]
            frame += 1
            counter += 3
        return distances

    # Writes the distances fo a file.
    @staticmethod
    def write(distances, filename_structure):
        print('+++++++++++++++' + filename_structure + '+++++++++++++++')
        if len(sys.argv) > 1:
            filename_out = filename_structure
        else:
            path = os.getcwd() + '/' + filename_structure
            filename_out = path + '/output/' + filename_structure + '_edges.txt'
            print('+++++++++++++++' + filename_out + '+++++++++++++++')
        with open(filename_out, 'w') as f_out:
            for d in distances:
                line = []
                for dd in d:
                    line.append(str(dd))
                line = ' '.join(line)
                line += '\n'
                f_out.write(line)
        return filename_out

    # Writes the end coordinates to a file. '_end_coords.txt'
    @staticmethod
    def writeEndCoordinates(distances, filename_structure):
        print('+++++++++++++++' + filename_structure + '+++++++++++++++')
        if len(sys.argv) > 1:
            filename_out = filename_structure
        else:
            path = os.getcwd() + '/' + filename_structure
            filename_out = path + '/output/' + filename_structure + '_end_coords.txt'
            print('+++++++++++++++' + filename_out + '+++++++++++++++')
        with open(filename_out, 'w') as f_out:
            for d in distances:
                line = []
                for dd in d:
                    line.append(str(dd))
                line = ' '.join(line)
                line += '\n'
                f_out.write(line)
        return filename_out




    # Main method in this script.
    @staticmethod
    def main(filename_structure, filename_trajectory, filename_particle_ids):
        #print('Edge Lengths')
        trajectory = EdgeLength.read(filename_trajectory) # trajectory file as first input argument
        particle_ids = EdgeLength.read(filename_particle_ids) # the list of edges in the second input argument
        particle_list = EdgeLength.splitParticles(particle_ids)
        distances = EdgeLength.getEndCoordinates(particle_list, trajectory)
        #for i, d in enumerate(distances):
        #    print(i, d[0])
        filename_edges = EdgeLength.writeEndCoordinates(distances, filename_structure)
        #writeOK(particle_list, trajectory)
        #print('END')
        return  filename_edges





if __name__ == '__main__':
    filename_trajectory = sys.argv[1]
    filename_particle_ids = sys.argv[2]
    filename_structure = 'edges_distances.txt'
    EdgeLength.main(filename_structure, filename_trajectory, filename_particle_ids)
