#   ServerMain.py
#       created on: 7 March 2017
#
#           Author: Andreas Gadin

from StartOxDNA import Analysis
from Directories import MakeDirectories
from InputFile import MakeInputFiles
import socket as socket
import time
import os

# To be able to iteratively improve structures with oxDNA this script has been made. It receives input files used in
# oxDNA from a master computer. It then simulates the  structure on a GPU, the user has to define how many GPUs that
# are available for simulation. The result from the simulations is then analysed with a error estimate of the total
# displacement of the nucleotides compared to the original structure. then the length of all the edges during the
# simulations are calculated and stored in files. The errors and the edgelength-files from the simulations are then
# sent back to the client.
# ----------------------------------------------------------------------------------------------------------------------#

# This is the main script run on the server side. It takes care of the communication with the client side and it calls
# functions that run the simulations and their analysis on the server.

# Takes a socket as input and waits for the client to send a bytestream of filenames. The received data is stored as a
# single string at first and then it is broken up with space(' ') as a delimiter and stores each individual filename in
# a list
def retrDirectorynames(s):
    data = s.recv(1024) # Stores the received data as string
    begin = time.time() # Takes a measurement of the current time
    # Starts the loop that controls that all data necessary har been received.
    while 1:
        # Checks if data has arrived and if a reasonable amount of time has passed. If it has the while loop breaks.
        if data and time.time() - begin > 1: #<--------------------------- This check is the wrong one, set the s.recv to wait for a timeout instead.
            break
        # If no data has arrived add more data
        data += s.recv(1024)
        time.sleep(0.1)
    # Removes the last delimiter and splits the string into smaller string and stores them in a list.
    directory_list = data.rstrip(' ').split(' ')
    s.close()
    return directory_list


# Receives a file from the client and saves it in a directory with it's own name that is placed in th current working
# directory.
def retrFile(s):
    path = os.getcwd() + '/' # Gets the search path to the current working directory
    filename = s.recv(1024)
    s.send('handshake') # sends a notification back to the client that the name of the file has been received
    data = s.recv(1024)
    filesize = int(data) # Receives the size of the file from the client
    s.send('handshake') # sends a notification back to the client that the filesize has been received

    # Checks that the ending of the file and sets the directory it shall go to correctly, this is needed because the
    # files with en bases are a bit of a special case.
    if filename.rfind('end_bases.txt') != -1:
        directory_name = filename[:filename.rfind('end_bases.txt')]
    else:
        directory_name = filename[:filename.rfind('.')]
    # opens a file to write to.
    with open(path + directory_name + '/input/' + filename, 'wb') as f:
        data = s.recv(1024)
        totalRecv = len(data)
        f.write(data)
        # Receives data as long as the data received is smaller than the file size, which has been retrieved from the
        # client.
        while totalRecv < filesize:
            data = s.recv(1024)
            totalRecv += len(data)
            f.write(data)
    # Closes the socket and the connection.
    s.close()

# Takes a socket and an absolute search path to the file intended amd sends it as a stream of bytes to the client.
def sendFile(s, filename):
    s.send(filename) # Sends filename to client
    handshake = s.recv(1024)  # waits for a note from the client telling the server that the filename has arrived.
    filesize = str(os.path.getsize(filename))
    s.send(filesize) # Sends file size to client
    handshake = s.recv(1024)  # waits for a note from the client telling the server that the file size has arrived.

    # Opens the file to send and reads it as bytes and sends it to the client.
    with open(filename, 'rb') as f:
        bytesToSend = f.read()
        s.send(bytesToSend)
        while bytesToSend != '':
            bytesToSend = f.read()
            s.send(bytesToSend)
    # Closes the socket and the connection.
    s.close()

# Main method that is supposed to run on the server.
def main():
    host = ''  # Accepts all hosts
    port = 5000  # Sets port the socket will be at
    s = socket.socket()  # Creates socket object
    s.bind((host, port))  # Sets the sockets potential host and which port it should listen to
    s.listen(1)  # Starts waiting for connection from client

    # Loop listens for client until user decides to end the program.
    while 1:
        print('server is listening...')

        # Retrieve the number of files used for the simulation
        c, addr = s.accept()  # Accepts connection from client and creates a new socket through, which they communicate
        print('Client ' + str(addr) + ' is connected!')
        number_of_files = int(c.recv(1024))
        c.close()
        # Retrieve file names/directory names
        c, addr = s.accept()  # Accepts connection from client and creates a new socket through, which they communicate
        print('Client ' + str(addr) + ' is connected!')
        # Calls the function that receives the file names (directory names).
        directoryname_list = retrDirectorynames(c)

        # Creates a new set of directories and input files for the oxDNA-simulations in the current working directory.
        for device_nr, filename in enumerate(directoryname_list):
            MakeDirectories.main(filename)
            MakeInputFiles.main(filename, device_nr)


        # Retrieve files .conf .top end_bases.txt files from  client and puts them in corresponding folders.
        for i in range(number_of_files):
            c, addr = s.accept()  # Waits for client connection
            print('Client ' + str(addr) + ' is connected')
            retrFile(c)  # Calls function that retrieves file from client
            print('Filetransfer ' + str(i + 1) + ' complete')
        # Simulates structures, analyses the simulations.
        displacement_errors, edgelength_filenames = Analysis.main(directoryname_list, int(number_of_files/3))
        print('Simulations and analysis complete')

        # Starts the process of sending back the results to the client.
        c, addr = s.accept()  # Waits for client connection
        # Sends errors and edgelength filenames back to client
        for error, filename in zip(displacement_errors, edgelength_filenames):
            filename = filename[filename.rfind('/') + 1:]  # Extracts just the filename from the absolute file path
            c.send(str(error) + ' ' + filename + ' ')  # Sends the error and filename back to client
        c.send('end')  # Sends end so that the client knows that this message is finished
        # Closes connection
        c.close()

        print('length of error and edgelength filename lists', len(displacement_errors), len(edgelength_filenames))
        # Sends edgelength files back to client.
        for filename in edgelength_filenames:
            c, addr = s.accept() # Waits for client connection
            print('Client ' + str(addr) + ' is connected!')
            sendFile(c, filename)
        # While-loop continues to listen for the client.

# Run if this script is the one that is invoked.
if __name__ == '__main__':
    main()