#   StartOxDNA.py
#       created on: 8 March 2017
#
#           Author: Andreas Gadin

from FitTrajectory import FitTrajectory
from EdgeLengths import EdgeLength
import threading
import subprocess
import os

# Global variables
global number_of_threads
global displacement_errors
global edgelength_filenames
number_of_threads = 2
displacement_errors = [0]*number_of_threads
edgelength_filenames = [0]*number_of_threads

class Analysis:

    @staticmethod
    def threadMethod(filename):
        print(threading.currentThread().getName())
        Analysis.oxDNA(filename)
        # Convert to .xyz?
        # error = Analysis.displacement(filename)
        # edgelength_filename = Analysis.edgelength(filename) # This analysis might be better to have on the client side
        # thread_name = threading.currentThread().getName()
        # thread_nr = int(thread_name[-1])
        # displacement_errors[thread_nr] = error
        # edgelength_filenames[thread_nr] = edgelength_filename

    @staticmethod
    def oxDNA(filename):
        oxDNA_directory = '/home/hogberg/oxDNA/build/bin/oxDNA'  # '/home/hogberg/oxDNA/bin/oxDNA' on node A and B

        cmd = [oxDNA_directory, os.getcwd() + '/' + filename + '/input/input_min']
        subprocess.call(cmd)
        cmd = [oxDNA_directory, os.getcwd() + '/' + filename + '/input/input_relax']
        subprocess.call(cmd)
        cmd = [oxDNA_directory, os.getcwd() + '/' + filename + '/input/input_go']
        subprocess.call(cmd)

    @staticmethod
    def displacement(filename):
        cmd = ['python', '/home/hogberg/oxDNA/UTILS/traj2vis.py', 'xyz', os.getcwd() + '/' + filename + '/output/min.conf',
               os.getcwd() + '/' + filename + '/input/' + filename + '.top']
        subprocess.call(cmd)
        cmd = ['python', '/home/hogberg/oxDNA/UTILS/traj2vis.py', 'xyz', os.getcwd() + '/' + filename + '/output/sim_traj.dat',
               os.getcwd() + '/' + filename + '/input/' + filename + '.top']
        subprocess.call(cmd)

        path = os.getcwd() + '/' + filename + '/output/'
        extension = '.xyz'
        FitTrajectory.main(filename, path + 'sim_traj.dat'+ extension, path + 'min.conf'+ extension) # gets the average structure
        error = FitTrajectory.main(filename, path + 'sim_traj.dat'+ extension, path + 'average_' + filename + extension) # gets the error of the trajectory compared to the average structure
        return error

    @staticmethod
    def edgelength(filename):
        path = os.getcwd() + '/' + filename
        edgelength_filename = EdgeLength.main(filename, path + '/output/' + 'sim_traj.dat', path + '/input/' + filename + 'end_bases.txt')
        return edgelength_filename

# Creates threads and starts analysis on every thread
    @staticmethod
    def main(filenames, number_of_threads):
        global displacement_errors
        global edgelength_filenames
        displacement_errors = [0] * number_of_threads
        edgelength_filenames = [0] * number_of_threads

        threads = []

        print('Starting threads')

        c = 0
        for i, filename in enumerate(filenames):
            t = threading.Thread(name='Thread' + str(i), target=Analysis.threadMethod,  kwargs={'filename': filename})
            threads.append(t)
            t.start()
            c += 1
            if c == number_of_threads:
                break
        c = 0
        for t in threads:
            t.join()
            c += 1
            if c == number_of_threads:
                break

        print('All threads have finished!')
        for i in range(len(filenames)):# (Erik) i moved the displacement analysis out of the threads as it seemed to eat up all the memory if it ran in parallell
            error = Analysis.displacement(filenames[i])
            edgelength_filename = Analysis.edgelength(filenames[i])  # This analysis might be better to have on the client side

            displacement_errors[i] = error
            edgelength_filenames[i] = edgelength_filename

            print 'finnished edge analysi for: ' + str(filenames[i])



        return displacement_errors, edgelength_filenames



if __name__ == '__main__':
    filenames = ['a.txt, b.txt, c.txt, d.txt']
    Analysis.main(filenames, 4)
